" Vim syntax file
" Language: gentle (friwi@first.fhg.de)
" (a language to write compilers in, similar to prolog, but much faster)
" Maintainer of syntax file: grude@tfh-berlin.de
" Version    of syntax file: 0.1
" -------------------------------------------------------------------
" For version 5.x: Clear all syntax items
" For version 6.x: Quit when a syntax file was already loaded
if version < 600
  syntax clear
elseif exists("b:current_syntax")
  finish
endif
" -------------------------------------------------------------------
" Gentle is mostly case sensitive:
syntax case match

" We don't need to look backwards to highlight correctly;
" this speeds things up greatly.
syntax sync minlines=1 maxlines=1
" -------------------------------------------------------------------
" Gentle keywords are enclosed in straight single quotes like this: '
" We therefore have to declare ' as an additional keyword character:
setlocal iskeyword+='

" Gentle keywords:
syntax keyword   gModule       'module'
syntax keyword   gModule       'MODULE'
syntax keyword   gUse          'use'
syntax keyword   gUse          'USE'
syntax keyword   gExport       'export'
syntax keyword   gExport       'EXPORT'
syntax keyword   gType         'type'
syntax keyword   gType         'TYPE'
syntax keyword   gVar          'var'
syntax keyword   gVar          'VAR'
syntax keyword   gTable        'table'
syntax keyword   gTable        'TABLE'
syntax keyword   gNonterm      'nonterm'
syntax keyword   gNonterm      'NONTERM'
syntax keyword   gToken        'token'
syntax keyword   gToken        'TOKEN'
syntax keyword   gPredCategory 'action'
syntax keyword   gPredCategory 'ACTION'
syntax keyword   gPredCategory 'condition'
syntax keyword   gPredCategory 'CONDITION'
syntax keyword   gPredCategory 'choice'
syntax keyword   gPredCategory 'CHOICE'
syntax keyword   gPredCategory 'sweep'
syntax keyword   gPredCategory 'SWEEP'
syntax keyword   gRule         'rule'  " 'rule'   marks the beginning of a rule
syntax keyword   gRule         'RULE'  " 'RULE'   marks the beginning of a rule
syntax match     gRule         "\."    " The dot. marks the end       of a rule
syntax keyword   gRoot         'root'
syntax keyword   gRoot         'ROOT'
" -------------------------------------------------------------------
" There are only 3 (+ 1) built in types in gentle:
syntax keyword   gBuiltInTypes INT STRING POS IDENT

" Arithmetic and other operators:
syntax match     gOperator  "[-+*/]" contained
syntax match     gOperator  "|"      contained " dirty solution for operators (| and |)
syntax match     gOperator  "->"     contained
syntax match     gOperator  "<-"     contained
syntax match     gOperator  ":"      contained

syntax match     gOperatorG  "[-+*/]"           " not contained
syntax match     gOperatorG  "->"               " not contained
syntax match     gOperatorG  "<-"               " not contained
syntax match     gOperatorG  ":"                " not contained
syntax match     gOperatorG  "\[|"              " not contained, clean solution
syntax match     gOperatorG  "|\]"              " not contained, clean solution

syntax keyword   gRelOperator lt le eq ne ge gt

" Some built in praedicates:
syntax keyword   gBuiltIns     print
syntax keyword   gBuiltIns     DefMeaning
syntax keyword   gBuiltIns     HasMeaning
" -------------------------------------------------------------------
" Important parts of a gentle programm are inside parentheses "(...)"
" Here we distinguish between the topmost parentheses and nested ones
" (but do not make any use of that distinction yet).

syntax region gParenthesis2 matchgroup=gParentheses start="(" end=")"ms=s+1 contained contains=@gInP2
syntax region gParenthesis1 matchgroup=gParentheses start="(" end=")"ms=s+1           contains=@gInP1

syntax match  gOtherName "\<[A-Za-z][A-Za-z0-9]*\>"   contained
syntax match  gParamName "\<[A-Za-z][A-Za-z0-9]*\s*:" contained

syntax cluster gInP2 contains=gParenthesis2,gString,gNumber,gArrows,gOperator
syntax cluster gInP1 contains=gParenthesis2,gParamName,gOtherName,gBuiltInTypes,gString,gNumber,gArrows,gOperator
" -------------------------------------------------------------------
" Comments

" TODO, FIXME and XXX (only highlighted in comments)
syntax keyword gTodo contained	TODO FIXME XXX

" There are 1 line comments (from "--" to the end of the line) and
" multiline comments (from "/*" to "*/"). Multiline comments may be nested,
" and the topmost comments will be highlighted differently from all nested comments.
" gCommentM1 comments may contain gCommentM2 comments
" gCommentM2 comments may contain gCommentM2 comments too.
" Without the match offset ms=s+2 there would be only comments of type gCommentM2 (because the
" topmost comment would contain itself as a contained comment).
" The match offset ms=s+2 causes the starting delimiter "/*" not to be highlighted. Therefore
" we have to highlight it separately with the matchgroup command.

syntax region gComment1L oneline contains=gTodo start="--"  end="$"

syntax region gCommentM2 matchgroup=gCommentM2  start="/\*" end="\*/"ms=s+2  contained contains=gCommentM2, gTODO
syntax region gCommentM1 matchgroup=gCommentM1  start="/\*" end="\*/"ms=s+2            contains=gCommentM2, gTODO
" -------------------------------------------------------------------
" Literals

" String literals (of type STRING):
syntax region  gString    start=/"/  skip=/\\"/  end=/"/

" integer literals (of type INT):
syntax match   gNumber    "-\=\<\d\+\>"
" -------------------------------------------------------------------
" MyHiLink is used here rather than HiLink to avoid conflicts with
" HiLink   in the syntax file help.vim (in case somebocy calls :help
" from within a gentle file highlighted with this file)
" -------------------------------------------------------------------
" Define the default highlighting.
" For version 5.7 and earlier: only when not done already
" For version 5.8 and later: only when an item doesn't have highlighting yet

if version >= 508 || !exists("did_gentle_syn_inits")
    if version < 508
        let did_gentle_syn_inits = 1
        command -nargs=+ MyHiLink hi link <args>
    else
        command -nargs=+ MyHiLink hi def link <args>
    endif

	MyHiLink gModule           Keyword
	MyHiLink gUse              Special
	MyHiLink gExport           Special 
	MyHiLink gType             Number
	MyHiLink gVar              Special
	MyHiLink gTable            Special
	MyHiLink gNonterm          Keyword
	MyHiLink gToken            Keyword
	MyHiLink gRule             Special
	MyHiLink gRoot             gRule       " root is a special rule
	MyHiLink gPredCategory     Type
	MyHiLink gRelOperator      Number

	"MyHiLink gParenthesis1     Special
	"MyHiLink gParenthesis2     Type
	MyHiLink gParentheses      Number
	MyHiLink gParamName        gCommentM1  " Parameter names are just comments
	MyHiLink gOtherName        White

	MyHiLink gBuiltInTypes     Type
	MyHiLink gOperator         Number
	MyHiLink gOperatorG        Number
	MyHiLink gBuiltIns         Operator
	MyHiLink gTodo             Todo

	MyHiLink gComment1L        Comment
	MyHiLink gCommentM1        Comment
	MyHiLink gCommentM2        Special
	MyHiLink gString           String
	MyHiLink gNumber           Number
	MyHiLink gError            Error

    delcommand MyHiLink
endif 

let b:current_syntax = "gentle"
" -------------------------------------------------------------------